/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.ukui.menu.xml -i dbus.h -a dbus-adaptor
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "dbus-adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class MenuAdaptor
 */

MenuAdaptor::MenuAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MenuAdaptor::~MenuAdaptor()
{
    // destructor
}

QString MenuAdaptor::GetSecurityConfigPath()
{
    // handle method call org.ukui.menu.GetSecurityConfigPath
    QString out0;
    QMetaObject::invokeMethod(parent(), "GetSecurityConfigPath", Q_RETURN_ARG(QString, out0));
    return out0;
}

void MenuAdaptor::ReloadSecurityConfig()
{
    // handle method call org.ukui.menu.ReloadSecurityConfig
    QMetaObject::invokeMethod(parent(), "ReloadSecurityConfig");
}

void MenuAdaptor::WinKeyResponse()
{
    // handle method call org.ukui.menu.WinKeyResponse
    QMetaObject::invokeMethod(parent(), "WinKeyResponse");
}

